;EasyCodeName=BlueSock,0
Include BlueSock.inc

.Const

.Data?

.Data
bWsaInitialized DB 0

.Code

BlueSockCleanup Proc Public
	Cmp bWsaInitialized, 0
	Jz @F
	Invoke WSACleanup
	.If Eax != 0
		Invoke ShowWSAError
	.EndIf
@@:
	Ret
BlueSockCleanup EndP

BlueSockConnect Proc Public lpBtAddr:DWord, lpGuidService:DWord
	Local sock:DWord
	Local sockAddrBth:SOCKADDR_BTH
	Lea Eax, sockAddrBth
	Mov sock, INVALID_SOCKET
	; AddressFamily
	Mov sockAddrBth.addressFamily, AF_BTH
	; BluetoothAddress
	Mov Edx, lpBtAddr
	m2m DWord Ptr [Eax + 2], [Edx + 2]
	m2m Word Ptr [Eax + 6], Word Ptr [Edx + 6]
	; clear last word of address in struct
	Mov Word Ptr [Eax + 8], 0
	; Serial Service GUID
	Mov Edx, lpGuidService
	m2m [Eax + 10], [Edx + 00]
	m2m [Eax + 14], [Edx + 04]
	m2m [Eax + 18], [Edx + 08]
	m2m [Eax + 22], [Edx + 12]
	; Port
	Mov sockAddrBth.port, 1
	; create a bluetooth socket
	Invoke socket, AF_BTH, SOCK_STREAM, BTHPROTO_RFCOMM
  Cmp Eax, INVALID_SOCKET
  Je proc_wsa_err
  Mov sock, Eax
  ; connect to the socket
	Invoke connect, sock, Addr sockAddrBth, SizeOf SOCKADDR_BTH
	Cmp Eax, 0
	Jne proc_wsa_err
	Jmp proc_exit

proc_wsa_err:
	Invoke ShowWSAError
	Jmp proc_err

proc_err:
	Cmp sock, INVALID_SOCKET
	Je @F
	Invoke closesocket, sock
	Mov sock, INVALID_SOCKET
@@:
	Jmp proc_exit

proc_exit:
	Mov Eax, sock
	Ret
BlueSockConnect EndP

BlueSockInitialize Proc Public
	; initialize winsock 2.2 (returns 2.0?)
	Local wsad:WSADATA
	Invoke  WSAStartup, 00202H, Addr wsad
  .If Eax != 0
  	Invoke ShowWSAError
		Xor Eax, Eax
	.Else
		Mov bWsaInitialized, 1
		Mov Eax, 1
	.EndIf
	Ret
BlueSockInitialize EndP

BlueSockSearchDevices Proc Public lpCallback:CALLBACK
	Local dwRet:DWord
	Local dwSize:DWord
	Local lpOut:LPVOID
	Local hLookup:HANDLE
	Local devInfo:BLUESOCK_DEVICEINFO
	Mov dwRet, 0
	Mov lpOut, NULL
	Mov hLookup, NULL
	; allocate a buffer for a device query
	Invoke LocalAlloc, LPTR, QUERY_BUFFER_SIZE
	Cmp Eax, 0
	Jz	proc_w32_err
	Mov lpOut, Eax
	; initialize the buffer as a WSAQUERYSET struct for bluetooth
	Assume Eax:Ptr WSAQUERYSET
	Mov [Eax].dwSize, SizeOf WSAQUERYSET
	Mov [Eax].dwNameSpace, NS_BTH
	Mov [Eax].lpBlob, NULL
	Assume Eax:Nothing
	; start the device search,
	Invoke WSALookupServiceBegin, lpOut, LUP_FLUSHCACHE Or LUP_CONTAINERS Or LUP_DEEP, Addr hLookup
	Cmp Eax, 0
	Jne proc_wsa_err
@@:
	Mov dwSize, QUERY_BUFFER_SIZE
	Invoke  WSALookupServiceNext, hLookup, LUP_RETURN_NAME Or LUP_RETURN_ADDR, Addr dwSize, lpOut
	.If Eax != 0
		; last one? then end
		Invoke WSAGetLastError
		Cmp Eax, WSA_E_NO_MORE
		Je @F
		Jmp proc_wsa_err
	.EndIf
	Mov Eax, lpOut
  m2m devInfo.btName, [Eax + 4] ; defInfo.btName = &lpOut->
	Mov Eax, [Eax + 48] ; ->lpcsaBuffer(CSADDR_INFO)
	; device address
 	Mov Eax, [Eax + 8] ; ->RemoteAddr.lpSockAddr
  Lea Ecx, devInfo
  m2m [Ecx + 0], [Eax + 0] ; .lpSockAddrLoDWord
  m2m [Ecx + 4], [Eax + 4] ; .lpSockAddrHiDWord
  Push Ecx
	Push 0
	Push 1
	Call lpCallback
	Jmp @B
@@:
	; return true
	Mov dwRet, 1
	Jmp proc_exit

proc_w32_err:
	Invoke ShowW32Error
	Jmp proc_exit

proc_wsa_err:
	Invoke ShowWSAError
	Jmp proc_exit

proc_exit:
	; cleanup
	.If lpOut != NULL
		Invoke GlobalFree, lpOut
	.EndIf
	.If hLookup != NULL
		Invoke WSALookupServiceEnd, hLookup
	.EndIf
	; return value
	Mov Eax, dwRet
	Ret
BlueSockSearchDevices EndP

ShowW32Error Proc Private
	Invoke MessageBox, NULL, LastError$(), SADD("BlueSock Win32 Error"), MB_ICONHAND
	Ret
ShowW32Error EndP

ShowWSAError Proc Private
	Local szSockError[1024]:Byte
	Local dwError:DWord
	Invoke WSAGetLastError
	Mov dwError, Eax
	Invoke FormatMessage,FORMAT_MESSAGE_FROM_SYSTEM,NULL,dwError,0,Addr szSockError,1024,NULL
	Invoke MessageBox,NULL,Addr szSockError,SADD("BlueSock Socket Error"), MB_ICONHAND
	Mov Eax, dwError
	Ret
ShowWSAError EndP
